severity = 4

# These modules set strict/warnings automatically
[TestingAndDebugging::RequireUseStrict]
equivalent_modules =

[TestingAndDebugging::RequireUseWarnings]
equivalent_modules =

# These policies we don't use
[-Miscellanea::RequireRcsKeywords]

# Allows us to set use strict;use warnings before package keyword
[-Modules::RequireExplicitPackage]
[Lax::RequireExplicitPackage::ExceptForPragmata]
